package com.link.base.datacomp.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.link.base.datacomp.dao.mybatis.mapper.DataCompMapper;
import com.link.base.datacomp.model.DataComp;
import com.link.core.basic.dao.mapper.BasicMapper;
import com.link.core.basic.interceptor.SecurityInterceptor;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicServiceImpl;
import com.link.core.objeccache.ObjectCache;
import com.link.core.util.PropertyUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.ReflectUtil;
import com.link.core.util.SpringUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import com.link.core.util.redisclient.RedisScanExec;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

@Service
public class DataCompServiceImpl extends BasicServiceImpl<DataComp> implements DataCompService, RedisScanExec {
   private static final Logger logger = LogManager.getLogger(DataCompServiceImpl.class);
   @Resource
   private DataCompMapper dataCompMapper;

   public BasicMapper<DataComp> getBasicMapper() {
      return this.dataCompMapper;
   }

   public void checkDiffByCache() throws Exception {
      LinkRedisClient linkRedisClient = null;

      try {
         linkRedisClient = RedisUtil.getJedis();
         List<String> configKeys = ObjectCache.getConfigKey();
         Iterator var3 = configKeys.iterator();

         while(var3.hasNext()) {
            String code = (String)var3.next();
            JSONObject configJsonObject = ObjectCache.getConfigByKey(code);
            String preRedisKey = configJsonObject.getString("preRedisKey");
            linkRedisClient.scanExec(500, preRedisKey.concat("*"), this, configJsonObject);
         }
      } catch (Exception var10) {
         Exception e = var10;
         logger.error(e.getMessage());
         throw e;
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

   }

   public void redisScanExec(List<String> stringList, JSONObject jsonObject) throws Exception {
      String queryCondition = jsonObject.getString("queryCondition");
      String code = jsonObject.getString("code");
      String[] queryConditionAttr = queryCondition.split(":");
      Iterator var6 = stringList.iterator();

      while(var6.hasNext()) {
         String str = (String)var6.next();
         Map<String, String> paramMap = new HashMap();
         String[] keyAttr = str.split(":");

         for(int i = 1; i < keyAttr.length; ++i) {
            paramMap.put(queryConditionAttr[i + 1], keyAttr[i]);
         }

         this.checkDiffSingleAndInsert(code, paramMap, (String)null);
      }

   }

   public DataComp checkDiffSingle(String key, Map<String, String> paramMap, String inputFields) throws Exception {
      LinkRedisClient redis = null;
      DataComp dataComp = null;

      DataComp var34;
      try {
         String micServiceName = ObjectCache.getConfigByKey(key).getString("micServiceName");
         Environment environment = (Environment)SpringUtil.getBean(Environment.class);
         String curMicServiceName = environment.getProperty("spring.application.name");
         if (!curMicServiceName.equalsIgnoreCase(micServiceName)) {
            DataComp var32 = this.queryDataComp(key, micServiceName, paramMap, inputFields);
            return var32;
         }

         redis = RedisUtil.getJedis();
         String queryField;
         if (StringUtils.isNotBlank(inputFields)) {
            queryField = inputFields;
         } else {
            queryField = ObjectCache.getConfigByKey(key).getString("queryField");
         }

         if (StringUtils.isBlank(queryField)) {
            throw new Exception("所维护的查询字段错误，请联系管理员！");
         }

         String[] fieldNameList = queryField.split(",");
         String queryCondition = ObjectCache.getConfigByKey(key).getString("queryCondition");
         if (StringUtils.isBlank(queryCondition)) {
            throw new Exception("所维护的查询条件错误，请联系管理员！");
         }

         String[] queryConditionAttr = queryCondition.split(":");
         String serviceName = queryConditionAttr[0];
         String modelName = queryConditionAttr[1];
         Class<BasicModel> clz = this.getModelClz(serviceName, modelName);
         Map<String, String> cacheStrMap = ObjectCache.queryFromCache(key, "cache", paramMap);
         List<BasicModel> cacheValueList = JSON.parseArray((String)cacheStrMap.get("value"), clz);
         Map databaseStrMap;
         if (cacheValueList == null || cacheValueList.size() < 1) {
            databaseStrMap = null;
            return databaseStrMap;
         }

         databaseStrMap = ObjectCache.queryFromCache(key, "database", paramMap);
         List<BasicModel> databaseValueList = JSON.parseArray((String)databaseStrMap.get("value"), clz);
         String coverFlag = PropertyUtil.getCropProperty("OBJ_CACHE_COVER_FLAG", 0L);
         String msg = "";
         if (cacheValueList.size() != databaseValueList.size()) {
            msg = "列表长度不相等！\r\n";
         }

         Iterator var22 = cacheValueList.iterator();

         while(var22.hasNext()) {
            BasicModel cacheObject = (BasicModel)var22.next();
            Object cacheIdValue = ReflectUtil.invokeGetterNew(cacheObject, "id");
            boolean matchFlag = false;
            Iterator var26 = databaseValueList.iterator();

            while(var26.hasNext()) {
               BasicModel databaseObject = (BasicModel)var26.next();
               Object databaseIdValue = ReflectUtil.invokeGetterNew(databaseObject, "id");
               if (cacheIdValue.toString().equals(databaseIdValue.toString())) {
                  msg = msg.concat(this.checkDataSame(databaseObject, cacheObject, fieldNameList));
                  matchFlag = true;
               }
            }

            if (!matchFlag) {
               msg = msg + "缓存的ID：" + cacheIdValue.toString() + "没有匹配上数据库记录\r\n";
            }
         }

         if (StringUtils.isNotBlank(msg)) {
            dataComp = this.setLineModel(0L, 0L, key, (String)cacheStrMap.get("value"), (String)databaseStrMap.get("value"), queryField, micServiceName, serviceName);
            if ("Y".equals(coverFlag)) {
               String redisKey = ObjectCache.genRedisKeyByConfCode(key, paramMap);
               Long expire = redis.ttl(redisKey);
               redis.set(redisKey, (String)databaseStrMap.get("value"));
               redis.expire(redisKey, Math.toIntExact(expire));
               dataComp.setStatus("Processed");
            } else {
               dataComp.setStatus("NotProcessed");
            }

            dataComp.setComments(msg);
         }

         var34 = dataComp;
      } finally {
         RedisUtil.returnResource(redis);
      }

      return var34;
   }

   private DataComp queryDataComp(String key, String micServiceName, Map<String, String> paramMap, String inputFields) throws Exception {
      HttpHeaders headers = new HttpHeaders();
      headers.add("INNERTOKEN", (String)SecurityInterceptor.internal_token.get());
      headers.add("LinkRequestId", (String)SecurityInterceptor.linkRequestId.get());
      headers.add("Authorization", "bearer " + (String)SecurityInterceptor.token.get());
      JSONObject jsonObject = new JSONObject();
      jsonObject.put("paramMap", JSONObject.toJSONString(paramMap));
      jsonObject.put("inputFields", inputFields);
      jsonObject.put("key", key);
      MultiValueMap<String, JSONObject> requestBody = new LinkedMultiValueMap();
      requestBody.add("jsonObject", jsonObject);
      HttpEntity<MultiValueMap<String, JSONObject>> request = new HttpEntity(requestBody, headers);
      RestTemplate restTemplate = (RestTemplate)SpringUtil.getBean(RestTemplate.class);
      String url = "http://" + micServiceName + "/link/dataCompensate/checkDiffSingle";
      ResponseEntity<JSONObject> response = restTemplate.postForEntity(url, request, JSONObject.class, new Object[0]);
      if (response.getStatusCode() != HttpStatus.OK) {
         throw new Exception("调用接口获取缓存报错！httpcode:" + response.getStatusCode());
      } else if (!((JSONObject)response.getBody()).getBoolean("success")) {
         throw new Exception(((JSONObject)response.getBody()).getString("result"));
      } else {
         DataComp retDataComp = (DataComp)((JSONObject)response.getBody()).get("result");
         return retDataComp;
      }
   }

   public void checkDiffSingleAndInsert(String key, Map<String, String> paramMap, String inputFields) throws Exception {
      DataComp dataComp = this.checkDiffSingle(key, paramMap, inputFields);
      if (dataComp != null) {
         Environment environment = (Environment)SpringUtil.getBean(Environment.class);
         String curMicServiceName = environment.getProperty("spring.application.name");
         if (!"link-base".equalsIgnoreCase(curMicServiceName)) {
            HttpHeaders headers = new HttpHeaders();
            headers.add("INNERTOKEN", (String)SecurityInterceptor.internal_token.get());
            headers.add("LinkRequestId", (String)SecurityInterceptor.linkRequestId.get());
            headers.add("Authorization", "bearer " + (String)SecurityInterceptor.token.get());
            HttpEntity<DataComp> request = new HttpEntity(dataComp, headers);
            RestTemplate restTemplate = (RestTemplate)SpringUtil.getBean(RestTemplate.class);
            String url = "http://link-base/link/dataCompensate/insert";
            ResponseEntity<JSONObject> response = restTemplate.postForEntity(url, request, JSONObject.class, new Object[0]);
            if (response.getStatusCode() != HttpStatus.OK) {
               throw new Exception("调用接口获取缓存报错！httpcode:" + response.getStatusCode());
            }

            if (!((JSONObject)response.getBody()).getBoolean("success")) {
               throw new Exception(((JSONObject)response.getBody()).getString("result"));
            }
         } else {
            this.dataCompMapper.insert(dataComp);
         }
      }

   }

   private String checkDataSame(BasicModel databaseValue, BasicModel cacheValue, String[] filedList) {
      String ret = "";
      String[] var5 = filedList;
      int var6 = filedList.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         String filed = var5[var7];
         Object databaseFiledValueObj = ReflectUtil.invokeGetterNew(databaseValue, filed);
         Object cacheFiledValueObj = ReflectUtil.invokeGetterNew(cacheValue, filed);
         String databaseFiledValue;
         String cacheFiledValue;
         if (databaseFiledValueObj != null && databaseFiledValueObj.getClass() == BigDecimal.class || cacheFiledValueObj != null && cacheFiledValueObj.getClass() == BigDecimal.class) {
            databaseFiledValue = databaseFiledValueObj == null ? "0" : ((BigDecimal)databaseFiledValueObj).stripTrailingZeros().toPlainString();
            cacheFiledValue = cacheFiledValueObj == null ? "0" : ((BigDecimal)cacheFiledValueObj).stripTrailingZeros().toPlainString();
         } else {
            databaseFiledValue = databaseFiledValueObj == null ? "null" : databaseFiledValueObj.toString();
            cacheFiledValue = cacheFiledValueObj == null ? "null" : cacheFiledValueObj.toString();
         }

         if (!databaseFiledValue.equals(cacheFiledValue)) {
            ret = ret.concat("ID：" + databaseValue.getId() + "：数据库值：" + databaseFiledValue + "；缓存的值：" + cacheFiledValue + "\r\n");
         }
      }

      return ret;
   }

   private Class<BasicModel> getModelClz(String serviceName, String modelName) throws Exception {
      Object service = SpringUtil.getBean(serviceName);
      String[] packageAttr = service.getClass().getName().split("\\$")[0].split("\\.");
      String wholeModelName = "";

      for(int i = 0; i < packageAttr.length - 2; ++i) {
         if (i == 0) {
            wholeModelName = wholeModelName.concat(packageAttr[i]);
         } else {
            wholeModelName = wholeModelName.concat(".").concat(packageAttr[i]);
         }
      }

      wholeModelName = wholeModelName.concat(".model.").concat(modelName);
      return Class.forName(wholeModelName);
   }

   private DataComp setLineModel(Long headId, Long dataId, String cacheKey, String cacheValue, String databaseValue, String fieldName, String source, String serviceName) {
      DataComp dataComp = new DataComp();
      dataComp.setHeadId(headId);
      dataComp.setCacheKey(cacheKey);
      dataComp.setCacheValue(cacheValue);
      dataComp.setDatabaseValue(databaseValue);
      dataComp.setDataId(dataId);
      dataComp.setFieldName(fieldName);
      dataComp.setSource(source);
      dataComp.setServiceName(serviceName);
      return dataComp;
   }
}
